// Import constants
// Note: In Chrome extension context, this file is directly included so no import needed
// These values will be accessible since constants.js is loaded first
// background.js

import {
  DEFAULT_OLLAMA_ENDPOINT,
  DEFAULT_MODEL_NAME,
  DEFAULT_ENABLED_SITES
} from './constants.js';  // Note: importing from constants.js now

import { detectLanguage } from './franc-module.js';



// Default values from constants
let ollamaEndpoint = DEFAULT_OLLAMA_ENDPOINT;
let modelName = DEFAULT_MODEL_NAME;

chrome.storage.sync.get(['ollamaEndpoint', 'modelName'], (result) => {
  if (result.ollamaEndpoint) {
    ollamaEndpoint = result.ollamaEndpoint;
  }
  if (result.modelName) {
    modelName = result.modelName;
  }
});


chrome.runtime.onInstalled.addListener(async details => {
  if (details.reason === "install") {
    // Set a flag in storage to indicate that it's the first run
    chrome.storage.local.set({ isFirstRun: true });
    const userId = self.crypto.randomUUID(); // Modern browsers support this
    
    // Store it in chrome.storage.local
    await chrome.storage.local.set({ userId });
    
    chrome.tabs.create({ url: chrome.runtime.getURL("help.html") });
  }
});


chrome.runtime.onStartup.addListener(() => {
  // Check if it's the first run and the help popup hasn't been shown yet
});



// Listen for command triggers
chrome.commands.onCommand.addListener((command) => {
  if (command === 'correct-text') {
    chrome.tabs.query({ active: true, currentWindow: true }, (tabs) => {
      if (tabs[0]) {
        chrome.tabs.sendMessage(tabs[0].id, { command: 'correct-text' });
      }
    });
  } else if (command === 'show-explanation') {
    chrome.tabs.query({ active: true, currentWindow: true }, (tabs) => {
      if (tabs[0]) {
        chrome.tabs.sendMessage(tabs[0].id, { command: 'show-explanation' });
      }
    });
  }
});

// Listen for messages from content script
chrome.runtime.onMessage.addListener((request, sender, sendResponse) => {
  if (request.action === 'store-data') {
    chrome.storage.session.set({
      [request.key]: request.data
    });
  } else if (request.action === 'correct-text') {
    correctText(request.text, request.type)
      .then(response => sendResponse(response))
      .catch(error => sendResponse({
        success: false,
        error: error.message
      }));
    return true; // Keep the message channel open for async response
  } else if (request.action === 'open-popup') {
    chrome.windows.create({
      url: 'popup/popup.html',
      type: 'popup',
      width: 500,
      height: 600
    });
  }
});

// background.js - updated correctText function
async function correctText(text, type) {
  try {
    let expLanguage = "English";
    try {
      // Get language from storage (default: "Auto")
      const result = await chrome.storage.local.get({ correctionLanguageText: "Auto" });
      expLanguage = result.correctionLanguageText;

      if (expLanguage === "Auto") {
        try {
          const languageCode = chrome.i18n.getUILanguage(); // e.g., "en-US", "fr-FR"
          const baseLanguageCode = languageCode.split('-')[0]; // e.g., "en" from "en-US"

          // Get the full language name (e.g., "English" instead of "English (United States)")
          const displayNames = new Intl.DisplayNames(['en'], { type: 'language' });
          expLanguage = displayNames.of(baseLanguageCode) || "English"; // Fallback if undefined

        } catch (error) {
          console.error("Failed to detect UI language:", error);
          expLanguage = "English"; // Fallback
        }
      }
    } catch (error) {
      console.error("Error in language detection:", error);
      expLanguage = "English"; // Final fallback
    }

    const userId = await getUserId();
    const textLanguage =  detectLanguage(text);

    // Make request to Ollama API
    //console.log("request: explanation language: ", expLanguage, ",  textLanguage: ", textLanguage.name, ", userId: ", userId, ", type: ", type, ", text: ", text);
    const response = await fetch(ollamaEndpoint, {
      method: 'POST',
      headers: {
        'Content-Type': 'application/json'
      },
      body: JSON.stringify({
        input: text,
        txtLang: textLanguage.name,
        expLang: expLanguage,
        type: type,
        userId: userId
      })
    });

    // Return error information if status is not 200
    if (!response.ok) {
      return {
        success: false,
        status: response.status,
        error: response.statusText || `API request failed with status: ${response.status}`
      };
    }

    const data = await response.json();
    const responseText = data.response.trim();

    // Extract the corrected text - looking strictly for the CORRECTED TEXT: section
    const correctedTextRegex = /CORRECTED TEXT:(?:\n|\r\n?)([\s\S]+?)(?=\n\s*CHANGES:|\n\s*\d+\.|\n\s*$)/i;
    const correctedTextMatch = responseText.match(correctedTextRegex);

    let correctedText = '';
    if (correctedTextMatch && correctedTextMatch[1]) {
      correctedText = correctedTextMatch[1].trim();
    } else {
      // Fallback: try other patterns
      const altMatch = responseText.match(/(?:Corrected Text:|Corrected:)(?:\n|\r\n?|\s)([\s\S]+?)(?=\n\s*(?:CHANGES:|Changes:|Explanations:|$))/i);
      if (altMatch && altMatch[1]) {
        correctedText = altMatch[1].trim();
      } else {
        // Last resort: take the first paragraph
        correctedText = responseText.split(/\n\s*\n/)[0].trim()
          .replace(/^(?:CORRECTED TEXT:|Corrected Text:|Corrected:)\s*/i, '');
      }
    }

    // Remove any quotes that might be present
    correctedText = correctedText.replace(/^["']|["']$/g, '').trim();

    // Extract changes/explanations section
    const changesRegex = /CHANGES:(?:\n|\r\n?)([\s\S]+)$/i;
    const changesMatch = responseText.match(changesRegex);

    let changesText = '';
    if (changesMatch && changesMatch[1]) {
      changesText = changesMatch[1].trim();
    } else {
      // Fallback: try other patterns
      const altMatch = responseText.match(/(?:Changes:|Explanations:)(?:\n|\r\n?|\s)([\s\S]+)$/i);
      if (altMatch) {
        changesText = altMatch[1].trim();
      }
    }

    // Format the changes text without parsing into objects
    // This keeps the original format as requested
    const corrections = changesText;

    return {
      success: true,
      correctedText: correctedText,
      corrections: corrections
    };
  } catch (error) {
    console.error('Error calling Ollama API:', error);
    return {
      success: false,
      error: error.message
    };
  }
}

// To get the user ID (in background.js or elsewhere)
async function getUserId() {
  const result = await chrome.storage.local.get(['userId']);
  
  // If no ID exists (shouldn't happen if onInstalled worked), generate one
  if (!result.userId) {
    const newId = self.crypto.randomUUID();
    await chrome.storage.local.set({ userId: newId });
    return newId;
  }
  
  return result.userId;
}
