// content.js

// Import MessengerHandler for platform detection and text insertion
(async () => {
  const { default: MessengerHandler, KNOWN_PLATFORMS } = await import(chrome.runtime.getURL('messengers.js'));

  // The rest of your content script code should go here
  // This ensures that your code only runs after the MessengerHandler is loaded

  // Example usage:
  const messenger = new MessengerHandler();
  // ... rest of your code

  // Constants for storage keys and classes
  const STORAGE_KEY_ORIGINAL = 'originalText';
  const STORAGE_KEY_CORRECTED = 'correctedText';
  const STORAGE_KEY_CORRECTIONS = 'corrections';
  const TOOLTIP_ID = 'correction-explanation-tooltip';
  const NOTIFICATION_ID = 'text-correction-notification';

  // Store corrections globally for tooltip display
  let globalCorrections = "";
  let lastCorrectedElement = null;
  let lastMousePosition = { x: 0, y: 0 };
  let messengerHandler = null;
  let isExplanationVisible = false;

  // Cache for corrections
  let lastCorrectedText = "";
  let lastCorrection = "";

  // Track API call status
  let apiCallInProgress = false;
  let pendingApiCall = null;

  // Initialize messenger handler
  document.addEventListener('DOMContentLoaded', () => {
    messengerHandler = new MessengerHandler();
  });

  // Track mouse position for tooltip placement
  document.addEventListener('mousemove', (e) => {
    lastMousePosition = { x: e.clientX, y: e.clientY };
  });

  // Listen for messages from the background script
  chrome.runtime.onMessage.addListener((message, sender, sendResponse) => {
    if (message.command === 'correct-text') {
      correctSelectedText(sendResponse); // Pass sendResponse for potential async use
      return true; // Indicate async response
    } else if (message.command === 'show-explanation') {
      showExplanation(sendResponse);
      return true; // Indicate async response
    }
    return false;
  });

  // Function to get selected text info
  function getSelectedTextInfo() {
    let selection = window.getSelection();
    let textToCorrect = '';
    let textElement = null;
    let selectionRange = null;

    if (selection && selection.toString().trim() !== '') {
      textToCorrect = selection.toString();
      selectionRange = selection.getRangeAt(0);
      textElement = selectionRange.commonAncestorContainer;
      if (textElement.nodeType === Node.TEXT_NODE) {
        textElement = textElement.parentElement;
      }
      return { text: textToCorrect, element: textElement, selection, range: selectionRange, hasSelection: true };
    } else {
      textElement = getActiveTextInput();
      if (textElement) {
        if (textElement.tagName === 'INPUT' || textElement.tagName === 'TEXTAREA') {
          textToCorrect = textElement.value || '';
        } else if (textElement.getAttribute('contenteditable') === 'true' || textElement.isContentEditable) {
          textToCorrect = textElement.textContent || '';
        } else {
          textToCorrect = textElement.textContent || '';
        }
      }
      return { text: "", element: textElement, selection: null, range: null, hasSelection: false };
    }
  }

  // Function to determine the active text input element
  function getActiveTextInput() {
    // Get the active element that can receive text input
    const activeElement = document.activeElement;

    // Check if the active element is a valid input element
    if (activeElement && (
      (activeElement.tagName === 'INPUT' && (activeElement.type === 'text' || activeElement.type === 'search')) ||
      activeElement.tagName === 'TEXTAREA' ||
      activeElement.getAttribute('contenteditable') === 'true' ||
      activeElement.isContentEditable
    )) {
      return activeElement;
    }

    // If no active element is found, look for common editable elements
    const potentialEditableElements = [
      // Common input selectors across various websites
      document.querySelector('[contenteditable="true"]'),
      document.querySelector('textarea'),
      document.querySelector('input[type="text"]'),
      // Specific selectors for known platforms
      document.querySelector('.input-message-input'),
      document.querySelector('.composer-input'),
      document.querySelector('.selectable-text')
    ];

    // Filter out null elements and return the first valid one
    const editableElements = potentialEditableElements.filter(el => el && el.isConnected !== false);
    return editableElements.length > 0 ? editableElements[0] : null;
  }

  function canProcessText(element, text, messengerHandler) {
    if (!element || !text || text.trim() === '') {
      return false;
    }

    // Check for Gmail subject fields
    if (messengerHandler.detectPlatform() === KNOWN_PLATFORMS.GMAIL) {
      if (messengerHandler.isGmailSubject(element)) {
        return false;
      }
    }

    // Check for Gmail subject fields
    if (messengerHandler.detectPlatform() === KNOWN_PLATFORMS.REDDIT) {
      //showNotification(`Can't replace the text on reddit. Use Alt+S`, 5000, element);
      return false;
    }

    // Check for non-editable static content (paragraphs, headings, etc.)
    const staticTags = ['P', 'H1', 'H2', 'H3', 'H4', 'H5', 'H6', 'SPAN', 'DIV', 'LI'];
    if (staticTags.includes(element.tagName) &&
      !element.isContentEditable &&
      element.getAttribute('contenteditable') !== 'true') {
      return false;
    }

    return true;
  }

  // Modified getCorrections function to ignore new requests when one is in progress
  async function getCorrections(text, type = 'correction') {
    // Check if we have this text cached
    if (lastCorrectedText === text) {
      return lastCorrection;
    }

    if (text && text.length > 2000) {
      throw new Error("The selected text is too big. Select a smaller one.");
    }
  
    // Check if API call is already in progress
    if (apiCallInProgress) {
      throw new Error("Another correction is already in progress");
    }
  
    // If not cached, make API call
    apiCallInProgress = true;
  
    try {
      const response = await chrome.runtime.sendMessage({
        action: 'correct-text',
        text: text,
        type: type
      });
  
      if (response && response.success) {
        // Cache the result
        const result = {
          correctedText: response.correctedText,
          corrections: response.corrections
        };
        lastCorrectedText = text;
        lastCorrection = result;
        return result;
      } else {
        // Handle API error with status and statusText
        const errorMessage = response.error || 'Unknown error';
        const errorStatus = response.status || 500;
        throw new Error(`${errorMessage} (${errorStatus})`);
      }
    } catch (error) {
      console.error('Error getting corrections:', error);
      throw error;
    } finally {
      apiCallInProgress = false;
    }
  }
  

  let correctionInProgress = false;

  async function correctSelectedText(sendResponse) {
  
    // Initialize messenger handler if not already done
    if (!messengerHandler) {
      messengerHandler = new MessengerHandler();
    }
  
    const textInfo = getSelectedTextInfo();
  
    console.log("correctSelectedText textInfo.text: ", textInfo.text)
    if (!textInfo.text || textInfo.text.trim() === '') {
      showNotification('No text found to correct.', 3000, textInfo.element);
      sendResponse({ success: false });
      return;
    }
  
    // First show the "please wait" notification if API is busy
    if (apiCallInProgress) {
      showNotification('Please wait for the current correction to complete', 3000, textInfo.element);
      // After 3 seconds, show the analyzing notification if still in progress
      setTimeout(() => {
        if (apiCallInProgress) {
          showNotification('Analyzing text', 0, textInfo.element);
        }
      }, 3000);
      sendResponse({ success: false, error: "Another correction is already in progress" });
      return;
    }
  
    showNotification('Analyzing text', 0, textInfo.element);
  
    try {
      correctionInProgress = true;
      
      // Get corrections from cache or API
      const correctionResult = await getCorrections(textInfo.text, 'correction');
  
      hideNotification();
  
      globalCorrections = correctionResult.corrections;
      lastCorrectedElement = textInfo.element;
  
      if (!canProcessText(textInfo.element, textInfo.text, messengerHandler)) {
        // Handle non-editable platforms (like Reddit)
        const currentTooltip = document.getElementById(`${TOOLTIP_ID}-correction`);
        const correctedText = correctionResult.correctedText;
        
        // Check if we should toggle (hide) the tooltip
        if (currentTooltip && currentTooltip.style.display !== 'none' && 
            lastCorrectedText === textInfo.text) {
          currentTooltip.style.display = 'none';
          tooltipState.correction.isVisible = false;
          tooltipState.correction.isPinned = false;
          sendResponse({ success: true });
          return;
        }
        
        // Otherwise show the tooltip
        showCorrectionPopup(correctedText, textInfo.element);
        lastCorrectedText = textInfo.text;
        sendResponse({ success: true });
        return;
      }
  
      // Handle editable platforms
      const insertSuccess = await messengerHandler.insertText(
        textInfo.element,
        correctionResult.correctedText,
        textInfo.range,
        textInfo.selection
      );
  
      if (insertSuccess) {
        sendResponse({ success: true });
      } else {
        console.error('Text insertion failed with messenger handler');
        showNotification('Failed to insert corrected text. Please try again.', 3000, textInfo.element);
        sendResponse({ success: false, error: 'Text insertion failed' });
      }
    } catch (error) {
      console.error('Error in text correction:', error);
      const errorMatch = error.message.match(/\((\d+)\)$/);
      const statusCode = errorMatch ? errorMatch[1] : null;
      
      // Only show error notification if it's not the "already in progress" message
      if (error.message !== "Another correction is already in progress") {
        showNotification(error.message, 5000, textInfo.element, true);
      }
      
      sendResponse({ success: false, error: error.message, status: statusCode });
    } finally {
      correctionInProgress = false;
    }
  }
  


  async function showExplanation(sendResponse) {
    const tooltip = document.getElementById(`${TOOLTIP_ID}-explanation`);
    const textInfo = getSelectedTextInfo();
    const currentState = tooltipState.explanation;
  
    // Toggle behavior - hide if visible and same text
    if (currentState.isVisible && tooltip && tooltip.style.display !== 'none') {
      const tooltipText = tooltip.querySelector('#tooltip-content-container')?.textContent || '';
      if (textInfo.text === lastCorrectedText || tooltipText.includes(textInfo.text)) {
        tooltip.style.display = 'none';
        currentState.isVisible = false;
        currentState.isPinned = false;
        isExplanationVisible = false;
        sendResponse({ success: true, action: 'hidden' });
        return;
      }
    }
  
 
    if (textInfo.hasSelection && textInfo.text.trim() !== '') {
      try {
        let correctionResult;
  
        if (lastCorrectedText === textInfo.text) {
          correctionResult = lastCorrection;
        } else {
          // First show the "please wait" notification if API is busy
          if (apiCallInProgress) {
            showNotification('Please wait for the current correction to complete', 3000, textInfo.element);
            // After 3 seconds, show the analyzing notification if still in progress
            setTimeout(() => {
              if (apiCallInProgress) {
                showNotification('Analyzing text', 0, textInfo.element);
              }
            }, 3000);
            throw new Error("Another correction is already in progress");
          }
  
          showNotification('Analyzing text', 0, textInfo.element);
          correctionResult = await getCorrections(textInfo.text, 'explanation');
          hideNotification();
        }
  
        if (correctionResult) {
          globalCorrections = correctionResult.corrections;
          lastCorrectedElement = textInfo.element;
          showExplanationPopup();
          isExplanationVisible = true;
          sendResponse({ success: true, action: 'shown' });
        } else {
          throw new Error("Failed to get corrections");
        }
      } catch (error) {
        if (error.message !== "Another correction is already in progress") {
          hideNotification();
          console.error('Error showing explanation:', error);
          showNotification(error.message, 5000, textInfo.element, true);
          sendResponse({ success: false, error: error.message });
        }
      }
    } else {
      if (lastCorrection.length > 0) {
        globalCorrections = lastCorrection;
        showExplanationPopup();
        isExplanationVisible = true;
        sendResponse({ success: true, action: 'shown' });
      } else if (globalCorrections) {
        showExplanationPopup();
        isExplanationVisible = true;
        sendResponse({ success: true, action: 'shown' });
      } else {
        showNotification('No corrections available. Please select text first.', 3000, textInfo.element);
        sendResponse({ success: false, message: 'No corrections available' });
      }
    }
  }
  


  function parseCorrectionItem(itemText) {
    if (!itemText.trim()) return null;

    const item = itemText.trim();
    const result = {
      number: '',
      original: '',
      corrected: '',
      explanation: '',
      notes: []
    };

    // Extract the numbered correction part
    const correctionMatch = item.match(/^(\d+)\.\s*"([^"]+)"\s*(?:->|→|-)\s*"([^"]+)"(:|$)/);
    if (correctionMatch) {
      result.number = correctionMatch[1];
      result.original = correctionMatch[2];
      result.corrected = correctionMatch[3];
    }

    // Extract the rest of the text after correction
    const remainingText = item.replace(/^\d+\..*?"[^"]*"\s*(?:->|→|-)\s*"[^"]*"(:|\s*)/, '').trim();

    // Handle both bracketed and non-bracketed explanations
    if (remainingText) {
      const bracketMatch = remainingText.match(/^\[([^\]]+)\]/);
      if (bracketMatch) {
        // Case 1: Text in square brackets
        result.explanation = bracketMatch[1].trim();
        result.notes.push(bracketMatch[1].trim());
      } else {
        // Case 2: Plain text explanation
        result.explanation = remainingText
          .replace(/^[|:—-]+\s*/, '')  // Remove leading punctuation
          .replace(/\s*[|:—-]+$/, '')  // Remove trailing punctuation
          .trim();
      }
    }

    return result;
  }

  function generateCorrectionHTML(correction) {
    if (!correction) return '';

    return `
      <div style="display: flex; margin-bottom: 6px;">
        <span style="color: #78a9d1; font-weight: bold; margin-right: 6px; font-size: 13px; flex-shrink: 0;">${correction.number}.</span>
        <div style="flex-grow: 1;">
          <div style="display: flex; flex-wrap: wrap; align-items: baseline; gap: 3px; margin-bottom: 2px;">
            <span style="color: #e17575; font-size: 13px;">"${correction.original}"</span>
            <span style="font-size: 12px;">→</span>
            <span style="color: #7acc7a; font-size: 13px;">"${correction.corrected}"</span>
          </div>
          <div style="color: #bbb5a0; font-size: 12.5px; line-height: 1.3;">
            ${correction.explanation.trim()}
          </div>
        </div>
      </div>
    `;
  }

  function processCorrections(correctionsText) {
    const correctionItems = correctionsText.split(/(?=^\d+\.)/m);
    const formattedItems = correctionItems.map(item => {
      const parsedItem = parseCorrectionItem(item);
      return generateCorrectionHTML(parsedItem);
    }).filter(Boolean);

    return formattedItems.join('<div style="height: 0.5px; background-color: rgba(255,255,255,0.1); margin: 4px 0;"></div>');
  }

  // Global notification element
  let notificationElement = null;
  let notificationDotsInterval;

  function showNotification(message, duration = 3000, relativeElement = null, isError = false) {
    // Clear any existing notification first
    if (notificationElement) {
      document.body.removeChild(notificationElement);
      notificationElement = null;
    }

    // Clear any existing interval
    if (notificationDotsInterval) {
      clearInterval(notificationDotsInterval);
      notificationDotsInterval = null;
    }

    notificationElement = document.createElement('div');
    notificationElement.id = NOTIFICATION_ID;
    notificationElement.textContent = message;

    // Set base styles
    let backgroundColor = 'rgba(0, 0, 0, 0.5)'; // Default background

    // Apply error styling if isError is true
    if (isError) {
      backgroundColor = 'rgba(255, 77, 77, 0.85)'; // Semi-transparent red for errors
    }

    notificationElement.style.cssText = `
      position: fixed;
      background-color: ${backgroundColor};
      color: white;
      padding: 10px 20px;
      border-radius: 5px;
      z-index: 10000;
      max-width: 300px;
      text-align: center;
      box-shadow: 0 2px 8px rgba(0, 0, 0, 0.2);
    `;
    document.body.appendChild(notificationElement);

    // Position the notification intelligently
    positionNotification(notificationElement, relativeElement);

    if (duration === 0) {
      // Create a fixed-width container for the dots
      notificationElement.textContent = message;

      // Add a span for the dots that has a fixed width
      const dotsSpan = document.createElement('span');
      dotsSpan.style.display = 'inline-block';
      dotsSpan.style.width = '24px'; // Width of 3 dots
      dotsSpan.style.textAlign = 'left';
      dotsSpan.textContent = '.';
      notificationElement.appendChild(dotsSpan);

      let dotCount = 1;
      notificationDotsInterval = setInterval(() => {
        // Check if notification element still exists before updating
        if (!notificationElement || !document.body.contains(notificationElement)) {
          clearInterval(notificationDotsInterval);
          notificationDotsInterval = null;
          return;
        }

        dotCount = (dotCount % 3) + 1;

        // Update just the dots span with proper spacing
        switch (dotCount) {
          case 1:
            dotsSpan.textContent = '.  '; // 1 dot + 2 spaces
            break;
          case 2:
            dotsSpan.textContent = '.. '; // 2 dots + 1 space
            break;
          case 3:
            dotsSpan.textContent = '...'; // 3 dots + 0 spaces
            break;
        }
      }, 500);
    } else if (duration > 0) {
      setTimeout(hideNotification, duration);
    }
  }

  // Hide the notification
  function hideNotification() {
    if (notificationElement && document.body.contains(notificationElement)) {
      document.body.removeChild(notificationElement);
      notificationElement = null;
    }

    // Always clear the interval when hiding
    if (notificationDotsInterval) {
      clearInterval(notificationDotsInterval);
      notificationDotsInterval = null;
    }
  }

  // Position notification relative to a specific element with intelligent placement
  function positionNotification(notification, relativeElement = null) {
    // Default position (center bottom) if no element is provided
    let posX = window.innerWidth / 2 - notification.offsetWidth / 2;
    let posY = window.innerHeight - notification.offsetHeight - 20;

    // If we have a specific element to position relative to
    if (relativeElement && document.body.contains(relativeElement)) {
      const elementRect = relativeElement.getBoundingClientRect();
      const elementCenterY = elementRect.top + (elementRect.height / 2);
      const elementCenterX = elementRect.left + (elementRect.width / 2);

      // 1. Position above when element is in bottom half of screen
      // 2. Position below when element is in top half of screen
      if (elementCenterY > window.innerHeight / 2) {
        // Bottom half - show above
        posY = elementRect.top - notification.offsetHeight - 10;
      } else {
        // Top half - show below
        posY = elementRect.bottom + 10;
      }

      // Always center horizontally relative to the selected text
      posX = elementCenterX - (notification.offsetWidth / 2);

      // Ensure notification stays within viewport boundaries
      posY = Math.max(10, posY); // Don't go above top of viewport
      posY = Math.min(posY, window.innerHeight - notification.offsetHeight - 10); // Don't go below bottom
      posX = Math.max(10, posX); // Don't go beyond left edge
      posX = Math.min(posX, window.innerWidth - notification.offsetWidth - 10); // Don't go beyond right edge
    }
    // Fall back to mouse position if no element is available
    else if (lastMousePosition.x !== 0 && lastMousePosition.y !== 0) {
      // Position above mouse if in bottom half, below if in top half
      if (lastMousePosition.y > window.innerHeight / 2) {
        posY = lastMousePosition.y - notification.offsetHeight - 20;
      } else {
        posY = lastMousePosition.y + 20;
      }

      // Center horizontally relative to mouse
      posX = lastMousePosition.x - notification.offsetWidth / 2;

      // Ensure it stays within viewport
      posY = Math.max(10, Math.min(posY, window.innerHeight - notification.offsetHeight - 10));
      posX = Math.max(10, Math.min(posX, window.innerWidth - notification.offsetWidth - 10));
    }

    // Apply the final position
    notification.style.left = `${posX}px`;
    notification.style.top = `${posY}px`;
  }

  // Enhanced tooltip positioning with scroll awareness and reliable dragging
  function positionTooltip(tooltip, element) {
    // Get current scroll position
    const scrollX = window.scrollX || window.pageXOffset;
    const scrollY = window.scrollY || window.pageYOffset;

    // Default position (center of viewport)
    let posX = (window.innerWidth / 2) - (tooltip.offsetWidth / 2) + scrollX;
    let posY = (window.innerHeight / 2) - (tooltip.offsetHeight / 2) + scrollY;

    // If we have a specific element to position relative to
    if (element && document.body.contains(element)) {
      const elementRect = element.getBoundingClientRect();
      const elementCenterY = elementRect.top + (elementRect.height / 2) + scrollY;
      const elementCenterX = elementRect.left + (elementRect.width / 2) + scrollX;

      // Position above/below based on element's position in viewport
      if (elementRect.top + (elementRect.height / 2) > window.innerHeight / 2) {
        // Bottom half - show above
        posY = elementRect.top + scrollY - tooltip.offsetHeight - 10;
      } else {
        // Top half - show below
        posY = elementRect.bottom + scrollY + 10;
      }

      // Center horizontally relative to the selected text
      posX = elementCenterX - (tooltip.offsetWidth / 2);

      // Ensure tooltip stays fully visible in viewport
      posY = Math.max(scrollY + 10, posY); // Don't go above top of viewport
      posY = Math.min(posY, scrollY + window.innerHeight - tooltip.offsetHeight - 10); // Don't go below bottom
      posX = Math.max(scrollX + 10, posX); // Don't go beyond left edge
      posX = Math.min(posX, scrollX + window.innerWidth - tooltip.offsetWidth - 10); // Don't go beyond right edge
    }
    // Fall back to mouse position if no element is available
    else if (lastMousePosition.x !== 0 && lastMousePosition.y !== 0) {
      // Position above mouse if in bottom half, below if in top half
      if (lastMousePosition.y > window.innerHeight / 2) {
        posY = lastMousePosition.y + scrollY - tooltip.offsetHeight - 20;
      } else {
        posY = lastMousePosition.y + scrollY + 20;
      }

      // Center horizontally relative to mouse
      posX = lastMousePosition.x + scrollX - (tooltip.offsetWidth / 2);

      // Ensure it stays within viewport
      posY = Math.max(scrollY + 10, Math.min(posY, scrollY + window.innerHeight - tooltip.offsetHeight - 10));
      posX = Math.max(scrollX + 10, Math.min(posX, scrollX + window.innerWidth - tooltip.offsetWidth - 10));
    }

    // Apply the final position
    tooltip.style.left = `${posX}px`;
    tooltip.style.top = `${posY}px`;
    tooltip.style.display = 'block';
  }

  // Unified tooltip system with all features
  const tooltipState = {
    correction: {
      lastPosition: { x: null, y: null },
      isVisible: false,
      isPinned: false,
      element: null,
      autoHideTimer: null
    },
    explanation: {
      lastPosition: { x: null, y: null },
      isVisible: false,
      isPinned: false,
      element: null,
      autoHideTimer: null
    }
  };

  function showUnifiedTooltip(options, type = 'correction') {
    const currentState = tooltipState[type];
    const tooltipId = `${TOOLTIP_ID}-${type}`;
    let tooltip = currentState.element || document.getElementById(tooltipId);

    // If tooltip is already visible and we have new content, just update it
    if (tooltip && currentState.isVisible) {
      const contentContainer = tooltip.querySelector('#tooltip-content-container');
      if (contentContainer) {
        contentContainer.innerHTML = options.content;

        // Reset the auto-hide timer if not pinned
        if (!currentState.isPinned && currentState.autoHideTimer) {
          clearTimeout(currentState.autoHideTimer);
          currentState.autoHideTimer = setTimeout(() => {
            if (tooltip.style.display !== 'none' && !currentState.isPinned) {
              const rect = tooltip.getBoundingClientRect();
              const scrollX = window.scrollX || window.pageXOffset;
              const scrollY = window.scrollY || window.pageYOffset;
              currentState.lastPosition = {
                x: rect.left + scrollX,
                y: rect.top + scrollY
              };
              tooltip.style.display = 'none';
              currentState.isVisible = false;
              if (options.onClose) options.onClose();
            }
          }, 30000);
        }

        return tooltip;
      }
    }

    // Calculate maximum height
    const viewportHeight = window.innerHeight;
    const maxTooltipHeight = Math.floor(viewportHeight * 0.75);

    // Create tooltip if doesn't exist
    if (!tooltip) {
      tooltip = document.createElement('div');
      tooltip.id = tooltipId;
      tooltip.style.cssText = `
          position: absolute;
          background-color: rgba(25, 25, 30, 0.92);
          color: rgb(224, 224, 224);
          padding: 14px;
          border-radius: 8px;
          font-size: 14px;
          max-width: 320px;
          max-height: ${maxTooltipHeight}px;
          box-shadow: rgba(0, 0, 0, 0.4) 0px 4px 15px;
          line-height: 1.4;
          z-index: 10001;
          user-select: none;
          margin: 0;
          transform: none;
          overflow: hidden;
          cursor: grab;
          display: none;
      `;
      document.body.appendChild(tooltip);
      currentState.element = tooltip;

      // Drag functionality
      let isDragging = false;
      let dragOffsetX = 0;
      let dragOffsetY = 0;

      const handleDragStart = (e) => {
        if (!e.target.closest('button')) {
          isDragging = true;
          const rect = tooltip.getBoundingClientRect();
          dragOffsetX = e.clientX - rect.left;
          dragOffsetY = e.clientY - rect.top;
          tooltip.style.cursor = 'grabbing';
          tooltip.style.userSelect = 'none';
          e.preventDefault();
        }
      };

      const handleDragMove = (e) => {
        if (isDragging) {
          const scrollX = window.scrollX || window.pageXOffset;
          const scrollY = window.scrollY || window.pageYOffset;

          let newX = e.clientX - dragOffsetX + scrollX;
          let newY = e.clientY - dragOffsetY + scrollY;

          tooltip.style.left = `${newX}px`;
          tooltip.style.top = `${newY}px`;

          currentState.lastPosition = { x: newX, y: newY };
        }
      };

      const handleDragEnd = () => {
        if (isDragging) {
          const rect = tooltip.getBoundingClientRect();
          const scrollX = window.scrollX || window.pageXOffset;
          const scrollY = window.scrollY || window.pageYOffset;

          currentState.lastPosition = {
            x: rect.left + scrollX,
            y: rect.top + scrollY
          };

          isDragging = false;
          tooltip.style.cursor = 'grab';
          tooltip.style.userSelect = '';
        }
      };

      tooltip.addEventListener('mousedown', handleDragStart);
      document.addEventListener('mousemove', handleDragMove);
      document.addEventListener('mouseup', handleDragEnd);
    }

    // Build tooltip content
    const buttonsHtml = `
        <button id="pin-tooltip" style="
            background: none;
            border: none;
            cursor: pointer;
            font-size: 16px;
            color: ${currentState.isPinned ? '#78a9d1' : '#777'};
            padding: 0;
            width: 20px;
            height: 20px;
            display: flex;
            align-items: center;
            justify-content: center;
        " title="${currentState.isPinned ? 'Unpin tooltip' : 'Pin tooltip'}">${currentState.isPinned ? '⏺' : '○'}</button>
        <button id="close-tooltip" style="
            background: none;
            border: none;
            cursor: pointer;
            font-size: 18px;
            color: #777;
            padding: 0;
            width: 20px;
            height: 20px;
            display: flex;
            align-items: center;
            justify-content: center;
        " title="Close tooltip">×</button>
    `;

    const copyButtonHtml = options.showCopyButton ? `
        <button id="copy-tooltip" style="
            background-color: transparent;
            border: 1px solid #78a9d1;
            border-radius: 4px;
            cursor: pointer;
            color: #78a9d1;
            padding: 4px 12px;
            font-size: 12px;
            display: flex;
            align-items: center;
            gap: 5px;
        " title="Copy text">
            <svg xmlns="http://www.w3.org/2000/svg" width="12" height="12" fill="currentColor" viewBox="0 0 16 16">
                <path d="M4 1.5H3a2 2 0 0 0-2 2V14a2 2 0 0 0 2 2h10a2 2 0 0 0 2-2V3.5a2 2 0 0 0-2-2h-1v1h1a1 1 0 0 1 1 1V14a1 1 0 0 1-1 1H3a1 1 0 0 1-1-1V3.5a1 1 0 0 1 1-1h1v-1z"/>
                <path d="M9.5 1a.5.5 0 0 1 .5.5v1a.5.5 0 0 1-.5.5h-3a.5.5 0 0 1-.5-.5v-1a.5.5 0 0 1 .5-.5h3zm-3-1A1.5 1.5 0 0 0 5 1.5v1A1.5 1.5 0 0 0 6.5 4h3A1.5 1.5 0 0 0 11 2.5v-1A1.5 1.5 0 0 0 9.5 0h-3z"/>
            </svg>
            Copy
        </button>
    ` : '';

    tooltip.innerHTML = `
        <div style="display: flex; justify-content: space-between; align-items: center; margin-bottom: 8px; padding: 0 4px;">
            <div style="font-weight: bold; font-size: 15px; color: #78a9d1;">
                ${options.title}
            </div>
            <div style="display: flex; gap: 6px; align-items: center;">
                ${buttonsHtml}
            </div>
        </div>
        <div id="tooltip-content-container" style="
            overflow-y: auto;
            max-height: ${maxTooltipHeight - 60}px;
            padding: 0 4px;
            margin: 0;
            line-height: 1.4;
            font-size: 13.5px;
        ">
            ${options.content}
        </div>
        ${options.showCopyButton ? `
        <div style="display: flex; justify-content: flex-end; margin-top: 6px;">
            ${copyButtonHtml}
        </div>
        ` : ''}
    `;

    // Position the tooltip
    tooltip.style.display = 'block';
    tooltip.style.visibility = 'hidden';

    const scrollX = window.scrollX || window.pageXOffset;
    const scrollY = window.scrollY || window.pageYOffset;
    const tooltipHeight = Math.min(tooltip.scrollHeight, maxTooltipHeight);

    if (currentState.lastPosition.x !== null && currentState.lastPosition.y !== null) {
      let newX = currentState.lastPosition.x;
      let newY = currentState.lastPosition.y;

      const tooltipTopRelative = newY - scrollY;
      const tooltipBottomRelative = tooltipTopRelative + tooltipHeight;

      const isPartiallyOut = tooltipTopRelative < 0 ||
        tooltipBottomRelative > viewportHeight;

      if (isPartiallyOut) {
        newY = scrollY + (viewportHeight / 2) - (tooltipHeight / 2);
      }

      tooltip.style.left = `${newX}px`;
      tooltip.style.top = `${newY}px`;
    } else {
      positionTooltip(tooltip, options.targetElement);
      const rect = tooltip.getBoundingClientRect();
      currentState.lastPosition = {
        x: rect.left + scrollX,
        y: rect.top + scrollY
      };
    }

    tooltip.style.visibility = 'visible';
    currentState.isVisible = true;

    // Add button event listeners
    const pinButton = tooltip.querySelector('#pin-tooltip');
    const closeButton = tooltip.querySelector('#close-tooltip');
    const copyButton = options.showCopyButton ? tooltip.querySelector('#copy-tooltip') : null;

    pinButton.addEventListener('click', (e) => {
      e.stopPropagation();
      currentState.isPinned = !currentState.isPinned;
      pinButton.innerHTML = currentState.isPinned ? '⏺' : '○';
      pinButton.title = currentState.isPinned ? 'Unpin tooltip' : 'Pin tooltip';
      pinButton.style.color = currentState.isPinned ? '#78a9d1' : '#777';

      // Clear auto-hide timer if pinned
      if (currentState.isPinned && currentState.autoHideTimer) {
        clearTimeout(currentState.autoHideTimer);
        currentState.autoHideTimer = null;
      }
    });

    closeButton.addEventListener('click', (e) => {
      e.stopPropagation();
      const rect = tooltip.getBoundingClientRect();
      currentState.lastPosition = {
        x: rect.left + scrollX,
        y: rect.top + scrollY
      };
      tooltip.style.display = 'none';
      currentState.isVisible = false;
      if (options.onClose) options.onClose();
    });

    // Add hover effects to buttons
    const buttons = [pinButton, closeButton];
    if (copyButton) buttons.push(copyButton);

    buttons.forEach(btn => {
      btn.addEventListener('mouseover', () => {
        btn.style.color = btn.id === 'copy-tooltip' ? '#a0c7e9' : '#aaa';
        btn.style.backgroundColor = 'rgba(255,255,255,0.1)';
      });
      btn.addEventListener('mouseout', () => {
        if (btn.id === 'pin-tooltip' && currentState.isPinned) {
          btn.style.color = '#78a9d1';
        } else if (btn.id === 'copy-tooltip') {
          btn.style.color = '#78a9d1';
        } else {
          btn.style.color = '#777';
        }
        btn.style.backgroundColor = 'transparent';
      });
    });

    // Add copy functionality if needed
    if (copyButton) {
      copyButton.addEventListener('click', () => {
        navigator.clipboard.writeText(options.rawText || options.content)
          .then(() => {
            const originalHTML = copyButton.innerHTML;
            copyButton.innerHTML = `
                        <svg xmlns="http://www.w3.org/2000/svg" width="12" height="12" fill="#4CAF50" viewBox="0 0 16 16">
                            <path d="M13.854 3.646a.5.5 0 0 1 0 .708l-7 7a.5.5 0 0 1-.708 0l-3.5-3.5a.5.5 0 1 1 .708-.708L6.5 10.293l6.646-6.647a.5.5 0 0 1 .708 0z"/>
                        </svg>
                        Copied!
                    `;

            setTimeout(() => {
              copyButton.innerHTML = originalHTML;
            }, 1500);
          })
          .catch(err => {
            console.error('Could not copy text: ', err);
            copyButton.style.color = '#ff4d4d';
            setTimeout(() => {
              copyButton.style.color = '#78a9d1';
            }, 1500);
          });
      });
    }

    // Add ESC key listener
    const handleKeyDown = (e) => {
      if (e.key === 'Escape' && currentState.isVisible && !currentState.isPinned) {
        const rect = tooltip.getBoundingClientRect();
        currentState.lastPosition = {
          x: rect.left + scrollX,
          y: rect.top + scrollY
        };
        tooltip.style.display = 'none';
        currentState.isVisible = false;
        if (options.onClose) options.onClose();
      }
    };
    document.addEventListener('keydown', handleKeyDown);

    // Auto-hide if not pinned
    if (!currentState.isPinned) {
      // Clear any existing timer
      if (currentState.autoHideTimer) {
        clearTimeout(currentState.autoHideTimer);
      }

      currentState.autoHideTimer = setTimeout(() => {
        if (tooltip.style.display !== 'none' && !currentState.isPinned) {
          const rect = tooltip.getBoundingClientRect();
          currentState.lastPosition = {
            x: rect.left + scrollX,
            y: rect.top + scrollY
          };
          tooltip.style.display = 'none';
          currentState.isVisible = false;
          if (options.onClose) options.onClose();
        }
      }, 30000);
    }

    return tooltip;
  }

  // Updated showExplanationPopup using the unified tooltip
  function showExplanationPopup() {
    if (!globalCorrections || globalCorrections.length === 0) {
      showNotification('No corrections available.', 3000);
      return;
    }

    if (!lastCorrectedElement) {
      showNotification('Cannot display explanations. Please correct text again.', 3000);
      return;
    }

    let content = typeof globalCorrections === 'string' ?
      processCorrections(globalCorrections) :
      'No corrections available.';

    showUnifiedTooltip({
      title: 'Corrections',
      content: content,
      targetElement: lastCorrectedElement,
      onClose: () => { tooltipState.explanation.isVisible = false; }
    }, 'explanation');
  }

  function showCorrectionPopup(text, targetElement = null) {
    showUnifiedTooltip({
        title: 'Corrected text',
        content: formatText(text),
        targetElement: targetElement,
        showCopyButton: true,
        rawText: text,
        onClose: () => { 
            tooltipState.correction.isVisible = false;
            tooltipState.correction.isPinned = false;
        }
    }, 'correction');
}

  // Helper functions remain the same
  function formatText(text) {
    if (!text) return '';
    const escapedText = text
      .replace(/&/g, '&amp;')
      .replace(/</g, '&lt;')
      .replace(/>/g, '&gt;')
      .replace(/"/g, '&quot;')
      .replace(/'/g, '&#039;');
    return escapedText.replace(/\n/g, '<br>');
  }
})();