// messengers.js - Universal messenger platform detection and handling
// This module provides intelligent platform detection and text insertion strategies

// Constants for known platforms and their characteristics
export const KNOWN_PLATFORMS = {
  DISCORD: 'discord',
  TELEGRAM: 'telegram',
  WHATSAPP: 'whatsapp',
  REDDIT: 'reddit',
  SLACK: 'slack',
  GMAIL: 'gmail',
  FACEBOOK_MESSENGER: 'facebook_messenger',
  YOUTUBE: 'youtube',
  UNKNOWN: 'unknown'
};

// Editor technologies
const EDITOR_TECHNOLOGIES = {
  LEXICAL: 'lexical',
  DRAFTJS: 'draftjs',
  QUILL: 'quill',
  CONTENTEDITABLE: 'contenteditable',
  INPUT: 'input',
  MONACO: 'monaco',
  CODEMIRROR: 'codemirror',
  UNKNOWN: 'unknown'
};

// Store successful strategies for domains
const DOMAIN_STRATEGIES_KEY = 'successful_insert_strategies';

// Class to handle messenger platform detection and text insertion
class MessengerHandler {
  constructor() {
    this.detectedPlatform = KNOWN_PLATFORMS.UNKNOWN;
    this.detectedTechnology = EDITOR_TECHNOLOGIES.UNKNOWN;
    this.domainStrategies = this.loadDomainStrategies();
    this.currentHostname = window.location.hostname;
  }

  // Load previously successful strategies for domains
  loadDomainStrategies() {
    try {
      const stored = localStorage.getItem(DOMAIN_STRATEGIES_KEY);
      return stored ? JSON.parse(stored) : {};
    } catch (e) {
      console.warn('Failed to load domain strategies:', e);
      return {};
    }
  }

  // Save successful strategy for current domain
  saveSuccessfulStrategy(strategy) {
    try {
      this.domainStrategies[this.currentHostname] = {
        strategy,
        timestamp: Date.now()
      };
      localStorage.setItem(DOMAIN_STRATEGIES_KEY, JSON.stringify(this.domainStrategies));
    } catch (e) {
      console.warn('Failed to save domain strategy:', e);
    }
  }

  isGmailSubject(element) {
    if (!element) return false;

    // 1. Direct detection of visible subject field
    const isVisibleSubject = (
      element.name === 'subjectbox' ||
      (element.id && /^:r[a-z0-9]+$/.test(element.id)) || // Dynamic IDs like :rq
      element.placeholder?.toLowerCase() === 'subject'
    );

    // 2. Detection of hidden subject field
    const isHiddenSubject = (
      element.name === 'subject' &&
      element.type === 'hidden' &&
      element.id && /^:[a-z]{2}$/.test(element.id) // IDs like :sb
    );

    // 3. Contextual verification (ensure we're in compose window)
    const inComposeWindow = (
      element.closest('form[id^=":"]') || // Form with dynamic ID
      element.closest('.bAs') || // Compose form class
      element.closest('.aoD.az6') // Subject container
    );

    return (isVisibleSubject || isHiddenSubject) && !!inComposeWindow;
  }

  // Detect the current platform based on URL and DOM structure
  detectPlatform() {
    const hostname = window.location.hostname;

    // Check for known platforms first
    if (hostname.includes('discord.com')) {
      this.detectedPlatform = KNOWN_PLATFORMS.DISCORD;
    } else if (hostname.includes('web.telegram.org')) {
      this.detectedPlatform = KNOWN_PLATFORMS.TELEGRAM;
    } else if (hostname.includes('web.whatsapp.com')) {
      this.detectedPlatform = KNOWN_PLATFORMS.WHATSAPP;
    } else if (hostname.includes('slack.com')) {
      this.detectedPlatform = KNOWN_PLATFORMS.SLACK;
    } else if (hostname.includes('gmail.com') || hostname.includes('mail.google.com')) {
      this.detectedPlatform = KNOWN_PLATFORMS.GMAIL;
    } else if (hostname.includes('facebook.com') || hostname.includes('messenger.com')) {
      this.detectedPlatform = KNOWN_PLATFORMS.FACEBOOK_MESSENGER;
    } else if (hostname.includes('youtube.com')) {
      this.detectedPlatform = KNOWN_PLATFORMS.YOUTUBE;
    } else if (hostname.includes('reddit.com')) {
      this.detectedPlatform = KNOWN_PLATFORMS.REDDIT;
    } else {
      this.detectedPlatform = KNOWN_PLATFORMS.UNKNOWN;

      // For unknown platforms, try to identify common patterns
      // Look for metadata in the page that might reveal the platform
      const metaTags = document.querySelectorAll('meta[name="application-name"], meta[property="og:site_name"]');
      for (const meta of metaTags) {
        const content = (meta.getAttribute('content') || '').toLowerCase();
        if (content.includes('telegram')) this.detectedPlatform = KNOWN_PLATFORMS.TELEGRAM;
        else if (content.includes('whatsapp')) this.detectedPlatform = KNOWN_PLATFORMS.WHATSAPP;
        else if (content.includes('discord')) this.detectedPlatform = KNOWN_PLATFORMS.DISCORD;
        else if (content.includes('slack')) this.detectedPlatform = KNOWN_PLATFORMS.SLACK;
        else if (content.includes('facebook') || content.includes('messenger')) this.detectedPlatform = KNOWN_PLATFORMS.FACEBOOK_MESSENGER;
      }

      // Also check for Facebook Messenger by looking for specific UI elements
      if (document.querySelector('[aria-label="Messenger"]') ||
        document.querySelector('[aria-label="Message"]') ||
        document.querySelector('[aria-label="Chat"]')) {
        this.detectedPlatform = KNOWN_PLATFORMS.FACEBOOK_MESSENGER;
      }
    }

    return this.detectedPlatform;
  }

  // Detect editor technology used by the platform
  detectEditorTechnology(element) {
    if (!element) return EDITOR_TECHNOLOGIES.UNKNOWN;

    // Check for Lexical editor (used by WhatsApp, Facebook Messenger)
    if (element.hasAttribute('data-lexical-editor') ||
      element.closest('[data-lexical-editor="true"]') ||
      document.querySelector('[data-lexical-editor="true"]') ||
      window.Lexical) {
      this.detectedTechnology = EDITOR_TECHNOLOGIES.LEXICAL;
    }
    // Check for Draft.js (used by older Facebook products)
    else if (element.closest('[data-contents="true"]') ||
      document.querySelector('[data-contents="true"][data-editor="true"]') ||
      window.Draft) {
      this.detectedTechnology = EDITOR_TECHNOLOGIES.DRAFTJS;
    }
    // Check for Quill editor (used by Slack and others)
    else if (element.classList.contains('ql-editor') ||
      element.closest('.ql-editor') ||
      document.querySelector('.ql-editor') ||
      window.Quill) {
      this.detectedTechnology = EDITOR_TECHNOLOGIES.QUILL;
    }
    // Check for Monaco editor (used in developer tools)
    else if (element.classList.contains('monaco-editor') ||
      element.closest('.monaco-editor') ||
      document.querySelector('.monaco-editor') ||
      window.monaco) {
      this.detectedTechnology = EDITOR_TECHNOLOGIES.MONACO;
    }
    // Check for CodeMirror (used in developer tools)
    else if (element.classList.contains('CodeMirror') ||
      element.closest('.CodeMirror') ||
      document.querySelector('.CodeMirror') ||
      window.CodeMirror) {
      this.detectedTechnology = EDITOR_TECHNOLOGIES.CODEMIRROR;
    }
    // Basic contentEditable elements
    else if (element.isContentEditable || element.getAttribute('contenteditable') === 'true') {
      this.detectedTechnology = EDITOR_TECHNOLOGIES.CONTENTEDITABLE;
    }
    // Standard input or textarea elements
    else if (element.tagName === 'INPUT' || element.tagName === 'TEXTAREA') {
      this.detectedTechnology = EDITOR_TECHNOLOGIES.INPUT;
    }
    else {
      this.detectedTechnology = EDITOR_TECHNOLOGIES.UNKNOWN;
    }

    return this.detectedTechnology;
  }

  // Get the element info with additional technology insights
  getElementInfo(element) {
    if (!element) return { type: 'unknown', tech: EDITOR_TECHNOLOGIES.UNKNOWN };

    const technology = this.detectEditorTechnology(element);
    let elementInfo = { type: 'unknown', tech: technology };

    // For input and textarea elements
    if (element.tagName === 'INPUT' || element.tagName === 'TEXTAREA') {
      const attributes = {};
      for (let i = 0; i < element.attributes.length; i++) {
        const attr = element.attributes[i];
        attributes[attr.name] = attr.value;
      }

      elementInfo = {
        type: element.tagName.toLowerCase(),
        tech: technology,
        attributes: attributes,
        parentNode: element.parentNode,
        nextSibling: element.nextSibling
      };
    }
    // For contentEditable elements
    else if (element.getAttribute('contenteditable') === 'true' || element.isContentEditable) {
      elementInfo = {
        type: 'contenteditable',
        tech: technology,
        isDiv: element.tagName === 'DIV',
        hasLexical: technology === EDITOR_TECHNOLOGIES.LEXICAL,
        hasDraftJs: technology === EDITOR_TECHNOLOGIES.DRAFTJS,
        hasQuill: technology === EDITOR_TECHNOLOGIES.QUILL
      };
    }
    else if (element.tagName) {
      elementInfo.type = element.tagName.toLowerCase();
    }

    return elementInfo;
  }

  // Insert text using the most appropriate method for the platform and editor
  async insertText(element, correctedText, selectionRange = null, selection = null) {
    if (this.isGmailSubject(element)) {
      return false; // This will exit IMMEDIATELY
    }

    if (!element) {
      return false;
    }

    // Early exit if we have a saved strategy blocking
    const domainStrategy = this.domainStrategies[this.currentHostname];
    if (domainStrategy?.strategy === 'BLOCK') {
      return false;
    }

    // Detect platform and technology if not already done
    if (this.detectedPlatform === KNOWN_PLATFORMS.UNKNOWN) {
      this.detectPlatform();
    }

    const elementInfo = this.getElementInfo(element);
    const strategiesToTry = this.determineStrategies(elementInfo);

    // NULL means complete block (like for Gmail subjects)
    if (strategiesToTry === null) {
      return false;
    }

    // Empty array means skip but try saved/default strategies
    if (Array.isArray(strategiesToTry) && strategiesToTry.length === 0) {
      // Continue to saved/default strategy attempt
    }

    // Try each strategy until one works
    for (const strategy of strategiesToTry) {
      const success = await this.tryInsertionStrategy(strategy, element, correctedText, selectionRange, selection);
      if (success) {
        // Save successful strategy
        this.saveSuccessfulStrategy(strategy);
        return true;
      }
    }

    // If all else fails, try a basic insertion as last resort
    return this.basicTextInsertion(element, correctedText);
  }

  determineStrategies(elementInfo) {
    const strategies = [];

    // Known platform-specific strategies
    if (this.detectedPlatform === KNOWN_PLATFORMS.WHATSAPP) {
      if (elementInfo.tech === EDITOR_TECHNOLOGIES.LEXICAL) {
        strategies.push('whatsapp-lexical', 'lexical-editor', 'execCommand', 'contentEditable');
      } else {
        strategies.push('contentEditable', 'execCommand');
      }
    }
    else if (this.detectedPlatform === KNOWN_PLATFORMS.TELEGRAM) {
      strategies.push('telegram', 'execCommand', 'contentEditable');
    }
    else if (this.detectedPlatform === KNOWN_PLATFORMS.DISCORD) {
      strategies.push('discord', 'react-contentEditable', 'contentEditable');
    }
    else if (this.detectedPlatform === KNOWN_PLATFORMS.SLACK) {
      if (elementInfo.tech === EDITOR_TECHNOLOGIES.QUILL) {
        strategies.push('quill-editor', 'contentEditable');
      } else {
        strategies.push('contentEditable', 'execCommand');
      }
    } else if (this.detectedPlatform === KNOWN_PLATFORMS.YOUTUBE) {
      strategies.push('youtube-contenteditable', 'contentEditable', 'execCommand');
    } else if (this.detectedPlatform === KNOWN_PLATFORMS.FACEBOOK_MESSENGER) {
      if (elementInfo.tech === EDITOR_TECHNOLOGIES.LEXICAL) {
        strategies.push('facebook-messenger', 'lexical-editor', 'contentEditable');
      } else {
        strategies.push('contentEditable', 'execCommand');
      }
    }
    else if (this.detectedPlatform === KNOWN_PLATFORMS.REDDIT) {
      strategies.push('reddit-lexical', 'lexical-editor', 'contentEditable');
    }
    else if (this.detectedPlatform === KNOWN_PLATFORMS.GMAIL) {
      // Gmail subject fields should block ALL strategies
      if (this.isGmailSubject(elementInfo.element)) {
        return null; // Special null value means "block completely"
      }
      strategies.push('contentEditable', 'execCommand');
    }
    // Technology-specific strategies for unknown platforms
    else {
      if (elementInfo.tech === EDITOR_TECHNOLOGIES.LEXICAL) {
        strategies.push('lexical-editor', 'react-contentEditable', 'contentEditable');
      }
      else if (elementInfo.tech === EDITOR_TECHNOLOGIES.DRAFTJS) {
        strategies.push('draftjs-editor', 'react-contentEditable', 'contentEditable');
      }
      else if (elementInfo.tech === EDITOR_TECHNOLOGIES.QUILL) {
        strategies.push('quill-editor', 'contentEditable');
      }
      else if (elementInfo.tech === EDITOR_TECHNOLOGIES.MONACO) {
        strategies.push('monaco-editor', 'programmatic-editor');
      }
      else if (elementInfo.tech === EDITOR_TECHNOLOGIES.CODEMIRROR) {
        strategies.push('codemirror-editor', 'programmatic-editor');
      }
      else if (elementInfo.type === 'contenteditable') {
        strategies.push('contentEditable', 'execCommand');
      }
      else if (elementInfo.type === 'input' || elementInfo.type === 'textarea') {
        strategies.push('input-value', 'execCommand');
      }
    }

    // Always add these fallbacks
    if (!strategies.includes('execCommand')) strategies.push('execCommand');
    if (!strategies.includes('contentEditable')) strategies.push('contentEditable');
    if (!strategies.includes('input-value')) strategies.push('input-value');
    if (elementInfo.type === 'input' || elementInfo.type === 'textarea') {
      strategies.unshift('input-value'); // Prioritize this for input elements
    }

    return strategies;
  }

  async tryInsertionStrategy(strategy, element, correctedText, selectionRange, selection) {
    // If strategy is empty array, skip correction
    if (Array.isArray(strategy) && strategy.length === 0) {
      return false;
    }
    try {
      switch (strategy) {
        case 'facebook-messenger':
          return this.insertIntoFacebookMessenger(element, correctedText, selectionRange, selection);
        case 'whatsapp-lexical':
          return this.insertIntoWhatsAppLexical(element, correctedText, selectionRange, selection);
        case 'lexical-editor':
          return this.insertIntoLexicalEditor(element, correctedText, selectionRange, selection);
        case 'draftjs-editor':
          return this.insertIntoDraftJsEditor(element, correctedText, selectionRange, selection);
        case 'quill-editor':
          return this.insertIntoQuillEditor(element, correctedText, selectionRange, selection);
        case 'reddit-lexical':
          return this.insertIntoRedditEditor(element, correctedText, selectionRange, selection);
        case 'gmail-subject':
          return this.insertIntoGmailSubject(element, correctedText);
        case 'youtube-contenteditable':
          return this.insertIntoYouTubeContentEditable(element, correctedText);
        case 'monaco-editor':
        case 'codemirror-editor':
        case 'programmatic-editor':
          return this.insertIntoProgrammaticEditor(element, correctedText);
        case 'telegram':
          return this.insertIntoTelegram(element, correctedText, selectionRange, selection);
        case 'discord':
          return this.insertIntoDiscord(element, correctedText);
        case 'react-contentEditable':
          return this.insertIntoReactContentEditable(element, correctedText);
        case 'contentEditable':
          return this.insertIntoContentEditable(element, correctedText, selectionRange, selection);
        case 'execCommand':
          return this.insertUsingExecCommand(element, correctedText);
        case 'input-value':
          return this.insertIntoInputValue(element, correctedText);
        default:
          console.warn(`Unknown strategy: ${strategy}`);
          return false;
      }
    } catch (error) {
      console.error(`Error with strategy ${strategy}:`, error);
      return false;
    }
  }

  // Basic text insertion as last resort
  basicTextInsertion(element, text) {
    try {
      // Try different approaches
      if (element.tagName === 'INPUT' || element.tagName === 'TEXTAREA') {
        element.value = text;
      } else if (element.isContentEditable || element.getAttribute('contenteditable') === 'true') {
        element.textContent = text;
      } else {
        element.innerText = text;
      }

      // Trigger generic events
      this.triggerGenericEvents(element);
      return true;
    } catch (error) {
      console.error('Basic text insertion failed:', error);
      return false;
    }
  }


  /**
   * Universal method to insert text with undo support
   * Uses the best available method for undo support on the current platform
   */
  async insertWithUndoSupport(element, text, selectionRange = null, selection = null) {

    // First try platform-specific undo-friendly methods
    if (this.detectedPlatform === KNOWN_PLATFORMS.DISCORD) {
      return this.insertIntoDiscord(element, text);
    }
    if (this.detectedPlatform === KNOWN_PLATFORMS.WHATSAPP) {
      return this.insertIntoWhatsAppLexical(element, text, selectionRange, selection);
    }
    if (this.detectedPlatform === KNOWN_PLATFORMS.FACEBOOK_MESSENGER) {
      return this.insertIntoFacebookMessenger(element, text, selectionRange, selection);
    }
    if (this.detectedPlatform === KNOWN_PLATFORMS.TELEGRAM) {
      // Use the specialized Telegram method that maintains undo history
      return this.insertIntoTelegramWithUndo(element, text, selectionRange, selection);
    }

    // The rest of the method remains unchanged...
    if (element.isContentEditable || element.getAttribute('contenteditable') === 'true') {
      try {
        return this.insertUsingExecCommand(element, text);
      } catch (e) {
      }
    }

    if (element.tagName === 'INPUT' || element.tagName === 'TEXTAREA') {
      return this.insertIntoInputWithUndo(element, text);
    }

    return this.insertText(element, text, selectionRange, selection);
  }

  /**
   * Specialized input insertion that preserves undo history
   */
  insertIntoInputWithUndo(element, text) {
    try {
      // Focus the element first
      element.focus();

      // Create and dispatch a 'keydown' event for Ctrl+A (select all)
      const selectAllEvent = new KeyboardEvent('keydown', {
        bubbles: true,
        cancelable: true,
        key: 'a',
        code: 'KeyA',
        ctrlKey: true,
        keyCode: 65,
        which: 65
      });
      element.dispatchEvent(selectAllEvent);

      // Use execCommand for undo support if available
      if (document.execCommand) {
        document.execCommand('insertText', false, text);
      } else {
        // Fallback to input method if execCommand isn't available
        const start = element.selectionStart;
        const end = element.selectionEnd;
        element.value = element.value.substring(0, start) + text + element.value.substring(end);
        element.selectionStart = element.selectionEnd = start + text.length;

        // Dispatch input event
        const inputEvent = new InputEvent('input', {
          bubbles: true,
          inputType: 'insertText',
          data: text
        });
        element.dispatchEvent(inputEvent);
      }

      return true;
    } catch (error) {
      console.error('Input with undo insertion failed:', error);
      return false;
    }
  }

  insertIntoWhatsAppLexical(element, text, selectionRange, selection) {
    try {
      // Focus on the element first
      element.focus();

      // Find the closest Lexical editor container
      const lexicalContainer = element.closest('[data-lexical-editor="true"]') || element;

      // Handle selection-based correction
      if (selection && !selection.isCollapsed && selectionRange) {
        // Delete the selected content
        selectionRange.deleteContents();

        // Insert the corrected text at cursor position
        const textNode = document.createTextNode(text);
        selectionRange.insertNode(textNode);

        // Reset selection to end of inserted text
        selection.removeAllRanges();
        const newRange = document.createRange();
        newRange.setStartAfter(textNode);
        newRange.setEndAfter(textNode);
        selection.addRange(newRange);
      } else {
        // Handle full text correction with proper span structure for WhatsApp
        const spanElement = lexicalContainer.querySelector('span[data-lexical-text="true"]');

        if (spanElement) {
          // Preserve existing class if it exists
          const originalClasses = spanElement.className;

          // Create a new span with the same attributes
          const newSpan = document.createElement('span');
          newSpan.setAttribute('data-lexical-text', 'true');
          newSpan.textContent = text;

          // Copy all original attributes
          for (let i = 0; i < spanElement.attributes.length; i++) {
            const attr = spanElement.attributes[i];
            if (attr.name !== 'data-lexical-text') {
              newSpan.setAttribute(attr.name, attr.value);
            }
          }

          // Replace the old span with the new one
          spanElement.parentNode.replaceChild(newSpan, spanElement);
        } else {
          // If span doesn't exist, find the paragraph and create proper structure
          const paragraph = lexicalContainer.querySelector('p');
          if (paragraph) {
            // Clear existing content
            paragraph.innerHTML = '';

            // Create appropriate WhatsApp text structure
            const newSpan = document.createElement('span');
            newSpan.setAttribute('data-lexical-text', 'true');
            newSpan.textContent = text;

            // Add common WhatsApp classes
            newSpan.className = 'selectable-text copyable-text';

            paragraph.appendChild(newSpan);
          } else {
            // Last resort: just set text content
            lexicalContainer.textContent = text;
          }
        }
      }

      // Trigger appropriate events for WhatsApp
      const inputEvent = new InputEvent('input', {
        bubbles: true,
        cancelable: true,
        inputType: 'insertText',
        data: text
      });
      lexicalContainer.dispatchEvent(inputEvent);

      const compositionEvent = new CompositionEvent('compositionend', {
        bubbles: true,
        cancelable: true,
        data: text
      });
      lexicalContainer.dispatchEvent(compositionEvent);

      lexicalContainer.dispatchEvent(new Event('change', { bubbles: true }));

      const keypressEvent = new KeyboardEvent('keypress', {
        bubbles: true,
        cancelable: true,
        key: 'Enter',
        keyCode: 13
      });
      lexicalContainer.dispatchEvent(keypressEvent);

      return true;
    } catch (error) {
      console.error('WhatsApp Lexical insertion failed:', error);
      return false;
    }
  }


  /**
   * Specialized Gmail Subject Field Insertion
   * Handles line breaks and hidden field synchronization
   */
  async insertIntoGmailSubject(element, correctedText) {
    try {

      return false;
    } catch (error) {
      console.error('Gmail subject insertion failed:', error);
      return false;
    }
  }

  // Add this method to the MessengerHandler class
  insertIntoFacebookMessenger(element, text, selectionRange, selection) {
    try {
      // Focus the element first
      element.focus();

      // Find the Lexical editor container
      const lexicalContainer = element.closest('[data-lexical-editor="true"]') || element;

      // Handle selection-based correction
      if (selection && !selection.isCollapsed && selectionRange) {
        // Delete the selected content
        selectionRange.deleteContents();

        // Insert the corrected text at cursor position
        const textNode = document.createTextNode(text);
        selectionRange.insertNode(textNode);

        // Reset selection to end of inserted text
        selection.removeAllRanges();
        const newRange = document.createRange();
        newRange.setStartAfter(textNode);
        newRange.setEndAfter(textNode);
        selection.addRange(newRange);
      } else {
        // Handle full text correction with proper span structure for Facebook Messenger
        const spanElement = lexicalContainer.querySelector('span[data-lexical-text="true"]');

        if (spanElement) {
          // Create a new span with the same attributes
          const newSpan = document.createElement('span');
          newSpan.setAttribute('data-lexical-text', 'true');
          newSpan.textContent = text;

          // Copy all original attributes and classes
          for (let i = 0; i < spanElement.attributes.length; i++) {
            const attr = spanElement.attributes[i];
            if (attr.name !== 'data-lexical-text') {
              newSpan.setAttribute(attr.name, attr.value);
            }
          }

          // Replace the old span with the new one
          spanElement.parentNode.replaceChild(newSpan, spanElement);
        } else {
          // If span doesn't exist, find the paragraph and create proper structure
          const paragraph = lexicalContainer.querySelector('p');
          if (paragraph) {
            // Clear existing content
            paragraph.innerHTML = '';

            // Create appropriate Facebook Messenger text structure
            const newSpan = document.createElement('span');
            newSpan.setAttribute('data-lexical-text', 'true');
            newSpan.textContent = text;

            // Add common Facebook Messenger classes
            newSpan.className = 'x3jgonx'; // This is Facebook's class for message text

            paragraph.appendChild(newSpan);
          } else {
            // Last resort: just set text content
            lexicalContainer.textContent = text;
          }
        }
      }

      // Trigger appropriate events for Facebook Messenger
      const inputEvent = new InputEvent('input', {
        bubbles: true,
        cancelable: true,
        inputType: 'insertText',
        data: text
      });
      lexicalContainer.dispatchEvent(inputEvent);

      const compositionEvent = new CompositionEvent('compositionend', {
        bubbles: true,
        cancelable: true,
        data: text
      });
      lexicalContainer.dispatchEvent(compositionEvent);

      lexicalContainer.dispatchEvent(new Event('change', { bubbles: true }));

      // Simulate a keypress to trigger any listeners
      this.simulateKeyPress(lexicalContainer);

      return true;
    } catch (error) {
      console.error('Facebook Messenger insertion failed:', error);
      return false;
    }
  }

  simulateKeyPress(element) {
    const keypressEvent = new KeyboardEvent('keypress', {
      bubbles: true,
      cancelable: true,
      key: 'Enter',
      keyCode: 13
    });
    element.dispatchEvent(keypressEvent);
  }

  // Generic Lexical editor insertion
  insertIntoLexicalEditor(element, text, selectionRange, selection) {
    try {
      // Focus the element
      element.focus();

      // Find the Lexical editor container
      const lexicalContainer = element.closest('[data-lexical-editor="true"]') || element;

      // Look for any text spans inside the editor
      const textSpans = lexicalContainer.querySelectorAll('[data-lexical-text="true"]');

      if (textSpans.length > 0) {
        // Replace text in the first span
        textSpans[0].textContent = text;
      } else {
        // If no spans, look for paragraphs
        const paragraphs = lexicalContainer.querySelectorAll('p');
        if (paragraphs.length > 0) {
          // Create a new text span
          const span = document.createElement('span');
          span.setAttribute('data-lexical-text', 'true');
          span.textContent = text;

          // Replace paragraph content
          paragraphs[0].innerHTML = '';
          paragraphs[0].appendChild(span);
        } else {
          // Last resort - set the container's content directly
          lexicalContainer.textContent = text;
        }
      }

      // Trigger events
      this.triggerReactEvents(lexicalContainer);
      return true;
    } catch (error) {
      console.error('Lexical editor insertion failed:', error);
      return false;
    }
  }

  insertIntoYouTubeContentEditable(element, correctedText, selectionRange, selection) {
    try {
      // Focus the element first
      element.focus();
      
      // Get current selection if not provided
      if (!selection) {
        selection = window.getSelection();
      }
      
      // Get selectionRange if not provided but selection exists
      if (!selectionRange && selection && selection.rangeCount > 0) {
        selectionRange = selection.getRangeAt(0);
      }
      
      // Check if we have a valid selection
      const hasSelection = selectionRange && selection && !selection.isCollapsed;
      
      if (hasSelection) {
        // Save the selection range to restore position later
        const rangeToReplace = selectionRange.cloneRange();
        
        // Select just that range to replace only selected text
        selection.removeAllRanges();
        selection.addRange(rangeToReplace);
        
        // Replace only the selected text with corrected text
        document.execCommand('insertText', false, correctedText);
      } else {
        // Option 2: Replace entire content      
        // Select all content
        const range = document.createRange();
        range.selectNodeContents(element);
        selection.removeAllRanges();
        selection.addRange(range);
        
        // Replace with corrected text (with undo support)
        document.execCommand('insertText', false, correctedText);
      }
      
      // Clear selection by collapsing it to the end
      if (selection) {
        selection.collapseToEnd();
      }
      
      // Trigger YouTube-specific events to ensure changes are recognized
      const inputEvent = new InputEvent('input', {
        bubbles: true,
        inputType: 'insertText',
        data: correctedText
      });
      element.dispatchEvent(inputEvent);
      element.dispatchEvent(new Event('change', { bubbles: true }));
      
      return true;
    } catch (error) {
      console.error('YouTube contenteditable insertion failed:', error);
      return false;
    }
  }


  /**
 * Enhanced Reddit Lexical Editor Text Insertion
 */
  async insertIntoRedditEditor(element, text) {
    try {
      // Step 1: Ensure we have the right element
      const editorElement = element.closest('[data-lexical-editor="true"]') ||
        document.querySelector('[data-lexical-editor="true"]');

      if (!editorElement) {
        console.error("Couldn't find Reddit's Lexical editor element");
        return false;
      }

      // Step 2: Focus the editor with some delay for stability
      editorElement.focus();
      await new Promise(resolve => setTimeout(resolve, 20));

      // Step 3: Try multiple insertion methods in sequence

      // Method 1: InputEvent approach
      const inputResult = await this.insertWithInputEvent(editorElement, text);
      if (inputResult) return true;

      // Method 2: Selection-based approach
      const selectionResult = await this.insertWithSelectionAPI(editorElement, text);
      if (selectionResult) return true;

      // Method 3: Direct DOM manipulation (most aggressive)
      const domResult = await this.insertWithDirectDOM(editorElement, text);
      if (domResult) return true;

      return false;
    } catch (error) {
      console.error('All Reddit insertion methods failed:', error);
      return false;
    }
  }

  /**
   * Try to insert text using InputEvent
   */
  async insertWithInputEvent(element, text) {
    try {
      // First clear any existing selection
      document.getSelection().removeAllRanges();

      // Focus the element again
      element.focus();

      // Create a beforeinput event
      const inputEvent = new InputEvent('beforeinput', {
        bubbles: true,
        cancelable: true,
        inputType: 'insertText',
        data: text
      });

      // Dispatch the event
      const success = element.dispatchEvent(inputEvent);

      // Follow up with an input event
      if (success) {
        element.dispatchEvent(new InputEvent('input', {
          bubbles: true,
          inputType: 'insertText',
          data: text
        }));

        // Add some post-processing events
        await this.triggerRedditEventSequence(element);

        // Check if text was inserted
        const currentText = element.textContent || '';
        return currentText.includes(text);
      }

      return false;
    } catch (error) {
      console.error('InputEvent insertion failed:', error);
      return false;
    }
  }

  /**
   * Try to insert text using Selection API
   */
  async insertWithSelectionAPI(element, text) {
    try {
      // First clear any existing selection
      document.getSelection().removeAllRanges();

      // Focus element
      element.focus();

      // Select all content within the editor
      const range = document.createRange();
      range.selectNodeContents(element);

      const selection = window.getSelection();
      selection.removeAllRanges();
      selection.addRange(range);

      // Try to use execCommand to insert the text
      const success = document.execCommand('insertText', false, text);

      if (success) {
        // Trigger necessary events
        await this.triggerRedditEventSequence(element);
        return true;
      }

      return false;
    } catch (error) {
      console.error('Selection API insertion failed:', error);
      return false;
    }
  }

  /**
   * Try direct DOM manipulation as a last resort
   */
  async insertWithDirectDOM(element, text) {
    try {
      // Step 1: Find the text span
      let textSpan = element.querySelector('[data-lexical-text="true"]');

      // If span doesn't exist, create one
      if (!textSpan) {
        const paragraph = element.querySelector('p') || document.createElement('p');
        paragraph.className = 'first:mt-0 last:mb-0';
        paragraph.setAttribute('dir', 'ltr');

        textSpan = document.createElement('span');
        textSpan.setAttribute('data-lexical-text', 'true');

        paragraph.appendChild(textSpan);

        if (!element.contains(paragraph)) {
          element.appendChild(paragraph);
        }
      }

      // Step 2: Replace the content
      textSpan.textContent = text;

      // Step 3: Force mutation events to trigger
      const observer = new MutationObserver(() => { });
      observer.observe(element, { childList: true, subtree: true, characterData: true });

      // Minor change to force mutation
      const tempSpan = document.createElement('span');
      tempSpan.style.display = 'none';
      element.appendChild(tempSpan);
      await new Promise(resolve => setTimeout(resolve, 10));
      element.removeChild(tempSpan);

      // Step 4: Trigger a comprehensive event sequence
      await this.triggerRedditEventSequence(element);

      // Disconnect observer
      observer.disconnect();

      return true;
    } catch (error) {
      console.error('Direct DOM manipulation failed:', error);
      return false;
    }
  }

  /**
   * Trigger a comprehensive event sequence for Reddit
   */
  async triggerRedditEventSequence(element) {
    // Create a sequence of events that mimics normal typing
    const events = [
      ['keydown', new KeyboardEvent('keydown', { key: 'a', ctrlKey: true, bubbles: true })],
      ['keyup', new KeyboardEvent('keyup', { key: 'a', ctrlKey: true, bubbles: true })],
      ['select', new Event('select', { bubbles: true })],
      ['selectstart', new Event('selectstart', { bubbles: true })],
      ['selectionchange', new Event('selectionchange', { bubbles: true })],
      ['compositionstart', new CompositionEvent('compositionstart', { bubbles: true })],
      ['compositionupdate', new CompositionEvent('compositionupdate', { bubbles: true })],
      ['compositionend', new CompositionEvent('compositionend', { bubbles: true })],
      ['input', new InputEvent('input', { bubbles: true, inputType: 'insertReplacementText' })],
      ['change', new Event('change', { bubbles: true })]
    ];

    // Dispatch events with small delays
    for (const [type, event] of events) {
      element.dispatchEvent(event);
      await new Promise(resolve => setTimeout(resolve, 5));
    }

    // Final change event
    setTimeout(() => {
      element.dispatchEvent(new Event('change', { bubbles: true }));
      element.dispatchEvent(new FocusEvent('blur', { bubbles: true }));
      setTimeout(() => {
        element.dispatchEvent(new FocusEvent('focus', { bubbles: true }));
      }, 10);
    }, 50);
  }


  // Draft.js editor insertion
  insertIntoDraftJsEditor(element, text, selectionRange, selection) {
    try {
      // Focus the element
      element.focus();

      // Find Draft.js container
      const draftContainer = element.closest('[data-contents="true"]') ||
        element.closest('[data-editor="true"]') ||
        element;

      // Look for text blocks
      const textBlocks = draftContainer.querySelectorAll('[data-block="true"]');

      if (textBlocks.length > 0) {
        // Find a text span
        const textSpan = textBlocks[0].querySelector('[data-text="true"]');
        if (textSpan) {
          textSpan.textContent = text;
        } else {
          // If no text span, set content to the block
          textBlocks[0].textContent = text;
        }
      } else {
        // Last resort
        draftContainer.textContent = text;
      }

      // Trigger React events
      this.triggerReactEvents(draftContainer);
      return true;
    } catch (error) {
      console.error('Draft.js editor insertion failed:', error);
      return false;
    }
  }

  // Quill editor insertion (used by Slack and others)
  insertIntoQuillEditor(element, text, selectionRange, selection) {
    try {
      // Find the Quill editor container
      const quillContainer = element.closest('.ql-editor') || element;

      // Focus the editor
      quillContainer.focus();

      // Handle selection-based correction
      if (selection && !selection.isCollapsed && selectionRange) {
        // Delete the selected content
        selectionRange.deleteContents();

        // Insert the corrected text at cursor position
        const textNode = document.createTextNode(text);
        selectionRange.insertNode(textNode);

        // Reset selection to end of inserted text
        selection.removeAllRanges();
        const newRange = document.createRange();
        newRange.setStartAfter(textNode);
        newRange.setEndAfter(textNode);
        selection.addRange(newRange);
      } else {
        // Full text replacement (original behavior)
        quillContainer.innerHTML = '';

        // Create a paragraph for the text
        const paragraph = document.createElement('p');
        paragraph.textContent = text;

        // Add the paragraph to the editor
        quillContainer.appendChild(paragraph);
      }

      // Trigger appropriate events
      const inputEvent = new InputEvent('input', {
        bubbles: true,
        inputType: 'insertText',
        data: text
      });
      quillContainer.dispatchEvent(inputEvent);

      const changeEvent = new Event('change', { bubbles: true });
      quillContainer.dispatchEvent(changeEvent);

      // Additional events Slack might expect
      const compositionEnd = new CompositionEvent('compositionend', {
        bubbles: true,
        data: text
      });
      quillContainer.dispatchEvent(compositionEnd);

      return true;
    } catch (error) {
      console.error('Quill editor insertion failed:', error);
      return false;
    }
  }

  // Professional code editor insertion (Monaco, CodeMirror)
  insertIntoProgrammaticEditor(element, text) {
    try {
      // Focus the element
      element.focus();

      // Try to identify if it's a monaco or codemirror instance
      const isMonaco = element.classList.contains('monaco-editor') || element.closest('.monaco-editor');
      const isCodeMirror = element.classList.contains('CodeMirror') || element.closest('.CodeMirror');

      if (isMonaco && window.monaco) {
        // Try to access the Monaco editor model - this is limited without direct API access
        const inputArea = element.querySelector('textarea');
        if (inputArea) {
          inputArea.value = text;
          inputArea.dispatchEvent(new Event('input', { bubbles: true }));
        }
      } else if (isCodeMirror && window.CodeMirror) {
        // Try to access CodeMirror instance - this is limited without direct API access
        const inputArea = element.querySelector('textarea');
        if (inputArea) {
          inputArea.value = text;
          inputArea.dispatchEvent(new Event('input', { bubbles: true }));
        }
      } else {
        // Generic approach for code editors
        const textArea = element.querySelector('textarea');
        if (textArea) {
          textArea.value = text;
          textArea.dispatchEvent(new Event('input', { bubbles: true }));
        } else {
          element.textContent = text;
        }
      }

      // Trigger generic input events
      this.triggerGenericEvents(element);
      return true;
    } catch (error) {
      console.error('Programmatic editor insertion failed:', error);
      return false;
    }
  }


  insertIntoTelegram(element, text, selectionRange, selection) {
    try {
      element.focus();

      if (element.tagName === 'TEXTAREA' || element.tagName === 'INPUT') {
        const start = element.selectionStart;
        const end = element.selectionEnd;
        const currentValue = element.value;
        const newValue = currentValue.substring(0, start) + text + currentValue.substring(end);

        // Simulate typing each character (simplistic and might not be ideal)
        for (let i = 0; i < text.length; i++) {
          const char = text[i];
          const keydownEvent = new KeyboardEvent('keydown', { key: char, char: char, code: `Key${char.toUpperCase()}`, bubbles: true, cancelable: true });
          element.dispatchEvent(keydownEvent);
          const keypressEvent = new KeyboardEvent('keypress', { key: char, char: char, code: `Key${char.toUpperCase()}`, bubbles: true, cancelable: true });
          element.dispatchEvent(keypressEvent);
        }

        element.value = newValue;
        element.selectionStart = element.selectionEnd = start + text.length;

        const inputEvent = new InputEvent('input', {
          bubbles: true,
          inputType: 'insertText',
          data: text
        });
        element.dispatchEvent(inputEvent);

        const changeEvent = new Event('change', { bubbles: true });
        element.dispatchEvent(changeEvent);
      } else if (element.isContentEditable || element.getAttribute('contenteditable') === 'true') {
        const sel = window.getSelection();
        const range = sel.getRangeAt(0);

        if (!sel.isCollapsed) {
          range.deleteContents();
        }

        const textNode = document.createTextNode(text);
        range.insertNode(textNode);

        const newRange = document.createRange();
        newRange.setStart(textNode, text.length);
        newRange.setEnd(textNode, text.length);
        sel.removeAllRanges();
        sel.addRange(newRange);

        // Try dispatching more granular events for contenteditable
        const inputEvent = new InputEvent('input', {
          bubbles: true,
          inputType: 'insertText',
          data: text
        });
        element.dispatchEvent(inputEvent);
      }

      return true;
    } catch (error) {
      console.error('Telegram insertion failed:', error);
      return false;
    }
  }


  insertIntoDiscord(element, text) {
    try {
      // Focus the element first
      element.focus();

      // Discord uses Slate.js editor with React
      const slateEditor = element.closest('[data-slate-editor="true"]') || element;

      // Get the current selection
      const selection = window.getSelection();
      const range = selection.getRangeAt(0);

      // For selection-based correction
      if (!selection.isCollapsed) {
        range.deleteContents();
        const textNode = document.createTextNode(text);
        range.insertNode(textNode);

        // Move cursor to end of inserted text
        selection.removeAllRanges();
        const newRange = document.createRange();
        newRange.setStart(textNode, text.length);
        newRange.setEnd(textNode, text.length);
        selection.addRange(newRange);
      }
      // For full text replacement
      else {
        // Find the text span
        const textSpan = slateEditor.querySelector('[data-slate-string="true"]');
        if (textSpan) {
          // Create a new text node to replace the content
          const newTextNode = document.createTextNode(text);

          // Replace the content while maintaining the slate structure
          if (textSpan.parentNode) {
            textSpan.parentNode.replaceChild(newTextNode, textSpan);
          }
        } else {
          // Fallback to direct content replacement
          slateEditor.textContent = text;
        }
      }

      // Trigger React-compatible events with proper data
      const inputEvent = new InputEvent('input', {
        bubbles: true,
        cancelable: true,
        inputType: 'insertText',
        data: text
      });
      slateEditor.dispatchEvent(inputEvent);

      // Additional events Discord might expect
      const compositionEnd = new CompositionEvent('compositionend', {
        bubbles: true,
        data: text
      });
      slateEditor.dispatchEvent(compositionEnd);

      // Force React to update its state
      const changeEvent = new Event('change', { bubbles: true });
      slateEditor.dispatchEvent(changeEvent);

      // Simulate blur/focus to trigger state update
      slateEditor.blur();
      slateEditor.focus();

      // Special handling for Discord's React implementation
      if (slateEditor._valueTracker) {
        slateEditor._valueTracker.setValue(text);
      }

      return true;
    } catch (error) {
      console.error('Discord insertion failed:', error);
      return false;
    }
  }

  // React-based contentEditable insertion
  insertIntoReactContentEditable(element, text) {
    try {
      // Focus the element first
      element.focus();

      // Set text content directly
      element.textContent = text;

      // Trigger React synthetic events
      this.triggerReactEvents(element);
      return true;
    } catch (error) {
      console.error('React contentEditable insertion failed:', error);
      return false;
    }
  }

  // Standard contentEditable insertion
  insertIntoContentEditable(element, text, selectionRange, selection) {
    try {
      // If we have a valid selection range, use it
      if (selection && selectionRange) {
        selectionRange.deleteContents();
        const textNode = document.createTextNode(text);
        selectionRange.insertNode(textNode);
        selection.removeAllRanges();
        selection.addRange(selectionRange);
      } else {
        // Otherwise, replace all content
        element.textContent = text;
      }

      // Trigger standard contentEditable events
      const eventTypes = ['input', 'change'];
      eventTypes.forEach(eventType => {
        const event = new Event(eventType, { bubbles: true, cancelable: true });
        element.dispatchEvent(event);
      });

      return true;
    } catch (error) {
      console.error('ContentEditable insertion failed:', error);
      return false;
    }
  }

  /**
   * Enhanced execCommand insertion with better event simulation
   */
  insertUsingExecCommand(element, text) {
    try {
      // Focus the element
      element.focus();

      // Select all existing text (creates an undo point)
      document.execCommand('selectAll', false, null);

      // Delete the selection (creates another undo point)
      document.execCommand('delete', false, null);

      // Insert new text (creates the final undo point)
      document.execCommand('insertText', false, text);

      // Simulate natural typing events
      this.simulateTypingEvents(element, text);

      return true;
    } catch (error) {
      console.error('ExecCommand insertion failed:', error);
      return false;
    }
  }


  /**
   * Simulates natural typing events to trigger proper undo behavior
   */
  simulateTypingEvents(element, text) {
    // Simulate keydown for each character
    for (let i = 0; i < text.length; i++) {
      const char = text[i];
      const keydownEvent = new KeyboardEvent('keydown', {
        bubbles: true,
        cancelable: true,
        key: char,
        charCode: char.charCodeAt(0),
        keyCode: char.charCodeAt(0),
        which: char.charCodeAt(0)
      });
      element.dispatchEvent(keydownEvent);

      const inputEvent = new InputEvent('input', {
        bubbles: true,
        cancelable: true,
        inputType: 'insertText',
        data: char
      });
      element.dispatchEvent(inputEvent);

      const keyupEvent = new KeyboardEvent('keyup', {
        bubbles: true,
        cancelable: true,
        key: char,
        charCode: char.charCodeAt(0),
        keyCode: char.charCodeAt(0),
        which: char.charCodeAt(0)
      });
      element.dispatchEvent(keyupEvent);
    }

    // Final change event
    const changeEvent = new Event('change', { bubbles: true });
    element.dispatchEvent(changeEvent);
  }


  // Input value insertion approach
  insertIntoInputValue(element, text) {
    try {
      if (element.tagName === 'INPUT' || element.tagName === 'TEXTAREA') {
        const start = element.selectionStart;
        const end = element.selectionEnd;

        if (typeof start === 'number' && typeof end === 'number') {
          const currentValue = element.value;
          element.value = currentValue.substring(0, start) + text + currentValue.substring(end);
          element.selectionStart = start + text.length;
          element.selectionEnd = start + text.length;
        } else {
          // If no selection, replace entire value
          element.value = text;
        }

        // Trigger input events
        this.triggerGenericEvents(element);
        return true;
      }
      return false;
    } catch (error) {
      console.error('Input value insertion failed:', error);
      return false;
    }
  }

  // Trigger React-compatible events
  triggerReactEvents(element) {
    // Input event 
    const inputEvent = new Event('input', { bubbles: true, cancelable: true });
    element.dispatchEvent(inputEvent);

    // React sometimes needs a native InputEvent
    const nativeInputEvent = new InputEvent('input', {
      bubbles: true,
      cancelable: true,
      inputType: 'insertText'
    });
    element.dispatchEvent(nativeInputEvent);

    // Trigger onChange
    const changeEvent = new Event('change', { bubbles: true, cancelable: true });
    element.dispatchEvent(changeEvent);

    // Simulated blur/focus to force React component updates
    element.blur();
    element.focus();

    // Check for React valueTracker and update if present
    if (element._valueTracker) {
      element._valueTracker.setValue('');
    }
  }

  // Trigger generic input events
  triggerGenericEvents(element) {
    const eventTypes = ['input', 'change'];
    eventTypes.forEach(eventType => {
      const event = new Event(eventType, { bubbles: true, cancelable: true });
      element.dispatchEvent(event);
    });

    // Try to trigger valueTracker updates for React
    if (element._valueTracker) {
      element._valueTracker.setValue('');
    }

    // Simulate focus to ensure update
    element.blur();
    element.focus();
  }

  // Verify if insertion was successful
  verifyInsertion(element, text) {
    // This is challenging to implement correctly without risking false positives/negatives
    // A basic approach is to check if the content matches or contains the text
    if (element.tagName === 'INPUT' || element.tagName === 'TEXTAREA') {
      return element.value.includes(text);
    } else {
      return element.textContent.includes(text) || element.innerText.includes(text);
    }
  }
}

// Export the messenger handler
export default MessengerHandler;